%% Compute final probabilities for reliability assessment
clear all
close all

% Histogram bins
bins = 400;

% Default values
fdi_false_alarm_fix = 0.01;
fdi_missed_detection_fix = 0.05;
servo_mtbf_fix = 588;

% Ranges
fdi_false_alarm_range = 0:0.0001:0.01;
fdi_missed_detection_range = 0:0.0001:0.01;
servo_mtbf_range = 588:7800;

% 1 for linear, 0 for logarithmic plot
plot_lin = 0;
%% Load data for config 1..3
load SIL_data_WP_turb.mat

% Cut actual pattern (10 turns and 10 straight paths)
start_sample = round(42.6*50);
stop_sample = round(627.54*50);

%% Get histograms (for whole data) - 584.94 s (SIL_data_WP), 583.34 s (SIL_data_WP_no_rudder)
[hist_val_a, hist_bin_a] = hist(r_aileron_pos(start_sample:stop_sample)*180/pi, bins);
[hist_val_e, hist_bin_e] = hist(r_elevator_pos(start_sample:stop_sample)*180/pi, bins);
[hist_val_r, hist_bin_r] = hist(b_rudder_pos(start_sample:stop_sample)*180/pi, bins);

%% Prob. in function of MTBF (config v1..3)

% Config v1, beta = any
range_e.min = -25;
range_e.max = -1;

range_r.min = -25;
range_r.max = 25;

range_a.min = -25;
range_a.max = 25;

prob_final = [];

for servo_mtbf = servo_mtbf_range
% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_r) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_r * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

figure(1)
if plot_lin == 1
    plot(servo_mtbf_range, prob_final, 'b')
else
    semilogy(servo_mtbf_range, prob_final, 'b')
end

xlabel('servo MTBF [hour]')
ylabel('prob. of catastrophic failure [1/hour]')
title(['servo MTBF changes, missed detection = ' num2str(fdi_missed_detection_fix) ...
    ', false alarm = ' num2str(fdi_false_alarm_fix)])
hold on

% Config v2, beta = any
range_e.min = -25;
range_e.max = 25;

range_r.min = -25;
range_r.max = 25;

range_a.min = -11;
range_a.max = 12;

prob_final = [];

for servo_mtbf = servo_mtbf_range

% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_r) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_r * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

if plot_lin == 1
    plot(servo_mtbf_range, prob_final, 'g')
else
    semilogy(servo_mtbf_range, prob_final, 'g')
end
xlabel('servo MTBF [hour]')
ylabel('prob. of catastrophic failure [1/hour]')

% Config v3, beta = any
range_e.min = -25;
range_e.max = -4;

range_r.min = -25;
range_r.max = 25;

range_a.min = -7;
range_a.max = 10;

prob_final = [];

for servo_mtbf = servo_mtbf_range

% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_r) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_r * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

if plot_lin == 1
    plot(servo_mtbf_range, prob_final, 'r')
else
    semilogy(servo_mtbf_range, prob_final, 'r')
end
xlabel('servo MTBF [hour]')
ylabel('prob. of catastrophic failure [1/hour]')

plot(servo_mtbf_range(1), 0, 'k')
legend('v1 (4s)', 'v2 (4s)', 'v3 (3s)', 'v4 (3s)')

%% Prob. in function of missed det. (config v1..3)

% Config v1, beta = any
range_e.min = -25;
range_e.max = -1;

range_r.min = -25;
range_r.max = 25;

range_a.min = -25;
range_a.max = 25;

prob_final = [];

for fdi_missed_detection = fdi_missed_detection_range
% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection + ...
    (1 - prob_final_r) * q * fdi_missed_detection + ...
    (1 - prob_final_a) * q * fdi_missed_detection + ...
    (1 - prob_final_a) * q * fdi_missed_detection;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_r * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

figure(2)
if plot_lin == 1
    plot(fdi_missed_detection_range, prob_final, 'b')
else
    semilogy(fdi_missed_detection_range, prob_final, 'b')
end
xlabel('missed detection rate')
ylabel('prob. of catastrophic failure [1/hour]')
title(['servo MTBF = ' num2str(servo_mtbf_fix), ' missed detection changes, false alarm = '...
    num2str(fdi_false_alarm_fix)])
hold on

% Config v2, beta = any
range_e.min = -25;
range_e.max = 25;

range_r.min = -25;
range_r.max = 25;

range_a.min = -11;
range_a.max = 12;

prob_final = [];

for fdi_missed_detection = fdi_missed_detection_range

% Prob. of cat. failure due to elevator
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of cat. failure due to rudder
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of cat. failure due to aileron
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection + ...
    (1 - prob_final_e) * q * fdi_missed_detection + ...
    (1 - prob_final_r) * q * fdi_missed_detection + ...
    (1 - prob_final_a) * q * fdi_missed_detection;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_r * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

if plot_lin == 1
    plot(fdi_missed_detection_range, prob_final, 'g')
else
    semilogy(fdi_missed_detection_range, prob_final, 'g')
end

% Config v3, beta = any
range_e.min = -25;
range_e.max = -4;

range_r.min = -25;
range_r.max = 25;

range_a.min = -7;
range_a.max = 10;

prob_final = [];

for fdi_missed_detection = fdi_missed_detection_range

% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection + ...
    (1 - prob_final_r) * q * fdi_missed_detection + ...
    (1 - prob_final_a) * q * fdi_missed_detection;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_r * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

if plot_lin == 1
    plot(fdi_missed_detection_range, prob_final, 'r')
else
    semilogy(fdi_missed_detection_range, prob_final, 'r')
end

plot(fdi_missed_detection_range(1), 0, 'k')
legend('v1 (4s)', 'v2 (4s)', 'v3 (3s)', 'v4 (3s)')
%% Prob. in function of false alarm (config v1..3)

% Config v1, beta = any
range_e.min = -25;
range_e.max = -1;

range_r.min = -25;
range_r.max = 25;

range_a.min = -25;
range_a.max = 25;

prob_final = [];

for fdi_false_alarm = fdi_false_alarm_range
% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_r) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm + ...
    (1 - q) * prob_final_r * fdi_false_alarm + ...
    (1 - q) * prob_final_a * fdi_false_alarm + ...
    (1 - q) * prob_final_a * fdi_false_alarm;

prob_final = [prob_final prob_final_all];
end

figure(3)
if plot_lin == 1
    plot(fdi_false_alarm_range, prob_final, 'b')
else
    semilogy(fdi_false_alarm_range, prob_final, 'b')
end
xlabel('false alarm rate [1/hour]')
ylabel('prob. of catastrophic failure [1/hour]')
title(['servo MTBF = ' num2str(servo_mtbf_fix) ', missed detection = '...
    num2str(fdi_missed_detection_fix) ', false alarm changes'])
hold on

% Config v2, beta = any
range_e.min = -25;
range_e.max = 25;

range_r.min = -25;
range_r.max = 25;

range_a.min = -11;
range_a.max = 12;

prob_final = [];

for fdi_false_alarm = fdi_false_alarm_range

% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_r) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm + ...
    (1 - q) * prob_final_e * fdi_false_alarm + ...
    (1 - q) * prob_final_r * fdi_false_alarm + ...
    (1 - q) * prob_final_a * fdi_false_alarm;

prob_final = [prob_final prob_final_all];
end

if plot_lin == 1
    plot(fdi_false_alarm_range, prob_final, 'g')
else
    semilogy(fdi_false_alarm_range, prob_final, 'g')
end

% Config v3, beta = any
range_e.min = -25;
range_e.max = -4;

range_r.min = -25;
range_r.max = 25;

range_a.min = -7;
range_a.max = 10;

prob_final = [];

for fdi_false_alarm = fdi_false_alarm_range

% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of rudder outside range
[prob_final_r, ~] = int_hist(range_r, hist_val_r, hist_bin_r);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_r * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_r) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm + ...
    (1 - q) * prob_final_r * fdi_false_alarm + ...
    (1 - q) * prob_final_a * fdi_false_alarm;

prob_final = [prob_final prob_final_all];
end

if plot_lin == 1
    plot(fdi_false_alarm_range, prob_final, 'r')
else
    semilogy(fdi_false_alarm_range, prob_final, 'r')
end

plot(fdi_false_alarm_range(1), 0, 'k')
legend('v1 (4s)', 'v2 (4s)', 'v3 (3s)', 'v4 (3s)')
%% Load data for config 4
clear range* *_sample hist* prob*
load SIL_data_WP_no_rudder_turb.mat

% Cut actual pattern (10 turns and 10 straight paths)
start_sample = round(42.6*50);
stop_sample = round(625.94*50);

%% Get histograms (for whole data)
[hist_val_a, hist_bin_a] = hist(r_aileron_pos(start_sample:stop_sample)*180/pi, bins);
[hist_val_e, hist_bin_e] = hist(r_elevator_pos(start_sample:stop_sample)*180/pi, bins);
[hist_val_r, hist_bin_r] = hist(b_rudder_pos(start_sample:stop_sample)*180/pi, bins);

%% Prob. in function of MTBF (config v4)

% Config v4, beta = any
range_e.min = -25;
range_e.max = -1;

range_a.min = -25;
range_a.max = 25;

prob_final = [];

for servo_mtbf = servo_mtbf_range
% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_a * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

figure(1)
if plot_lin == 1
    plot(servo_mtbf_range, prob_final, 'k')
else
    semilogy(servo_mtbf_range, prob_final, 'k')
end

%% Prob. in function of missed det. (config v4)

% Config v4, beta = any
range_e.min = -25;
range_e.max = -1;

range_a.min = -25;
range_a.max = 25;

prob_final = [];

for fdi_missed_detection = fdi_missed_detection_range
% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_a * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection + ...
    (1 - prob_final_a) * q * fdi_missed_detection + ...
    (1 - prob_final_a) * q * fdi_missed_detection;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix + ...
    (1 - q) * prob_final_a * fdi_false_alarm_fix;

prob_final = [prob_final prob_final_all];
end

figure(2)
if plot_lin == 1
    plot(fdi_missed_detection_range, prob_final, 'k')
else
    semilogy(fdi_missed_detection_range, prob_final, 'k')
end

%% Prob. in function of false alarm (config v4)

% Config v4, beta = any
range_e.min = -25;
range_e.max = -1;

range_a.min = -25;
range_a.max = 25;

prob_final = [];

for fdi_false_alarm = fdi_false_alarm_range
% Prob. of elevator outside range
[prob_final_e, ~] = int_hist(range_e, hist_val_e, hist_bin_e);

% Prob. of aileron outside range
[prob_final_a, ~] = int_hist(range_a, hist_val_a, hist_bin_a);

% Servo failure rate
q = 1 / servo_mtbf_fix;

% Summarize prob. by control surfaces
prob_final_all = prob_final_e * q + ...
    prob_final_a * q + ...
    prob_final_a * q;

% Include FDI performance - missed detection rate
prob_final_all = prob_final_all + ... 
    (1 - prob_final_e) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix + ...
    (1 - prob_final_a) * q * fdi_missed_detection_fix;

% Include FDI performance - false alarm rate
prob_final_all = prob_final_all + ... 
    (1 - q) * prob_final_e * fdi_false_alarm + ...
    (1 - q) * prob_final_a * fdi_false_alarm + ...
    (1 - q) * prob_final_a * fdi_false_alarm;

prob_final = [prob_final prob_final_all];
end

figure(3)
if plot_lin == 1
    plot(fdi_false_alarm_range, prob_final, 'k')
else
    semilogy(fdi_false_alarm_range, prob_final, 'k')
end
